﻿//
//	METC.cs		三菱ETC通信クラス
//				Copyright(C)2011-2012 Kashi Electronics Designs
//				kashima@kaele.com (Hitoshi Kashima)
//	2011/12/28	初版
//	2012/1/5	Ver1.0
//
using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.IO;
using System.IO.Ports;
using System.Windows.Forms;

/// <summary>
/// 三菱ETC通信クラス
/// </summary>
public class METC : Object
{
	/// <summary>読み出された機器番号</summary>
	public String machineID;
	/// <summary>読み出された型式番号</summary>
	public String classID;
	/// <summary>読み出されたモデル名</summary>
	public String modelName;
	/// <summary>読み出されたセットアップ日付</summary>
	public DateTime setupDate;
	/// <summary>最後にETCStatus.PaidInfoを受けた時の支払金額</summary>
	public int LastCost;
	/// <summary>ETCのイベントにより発生します</summary>
	public event METCEventHandler Event;
	/// <summary>読み出された履歴情報</summary>
	public ETCHistoryData HistoryData;

	/// <summary>
	/// シリアルポートが開いているかどうかの値
	/// </summary>
	public bool IsOpen
	{
		get { return serial.IsOpen; }
	}

	/// <summary>METCクラスイベントハンドラー</summary>
	/// <param name="sender"></param>
	/// <param name="f">発生ステータス</param>
	/// <param name="val">付帯値 ETCStatus.PaidInfoの時、料金値</param>
	public delegate void METCEventHandler(object sender, ETCStatus f, int val);

	private SerialPort serial;
	private volatile ETCStatus LastReceivedFrame;
	private volatile bool LinkUP = false;
	private System.Timers.Timer timer;
	private volatile int timeout = 0;
	private StreamWriter logfile;


	/// <summary>受信フレームタイプ</summary>
	public enum ETCStatus : int
	{
		/// <summary>ハートビート受信</summary>
		HeartBeat,
		/// <summary>通信可能</summary>
		CommOK,
		/// <summary>通信許可</summary>
		Permit,
		/// <summary>カードエラー</summary>
		CardError,
		/// <summary>読込エラー</summary>
		Error05,
		/// <summary>カード挿入</summary>
		CardInsert,
		/// <summary>カード取り出し</summary>
		CardRemove,
		/// <summary>出口ゲート通過</summary>
		GateOut,
		/// <summary>入口ゲート通過</summary>
		GateIn,
		/// <summary>ゲート通過エラー</summary>
		GateErr,
		/// <summary>お知らせアンテナ通過</summary>
		Antenna,
		/// <summary>未挿入警告</summary>
		Error01,
		/// <summary>アライブ通知</summary>
		Alive,
		/// <summary>履歴あり</summary>
		HistAvail,
		/// <summary>継続する履歴フレームなし</summary>
		NoMore,
		/// <summary>カードなし</summary>
		NoCard,
		/// <summary>カードあり</summary>
		OnCard,
		/// <summary>機器番号フレーム</summary>
		MachineID,
		/// <summary>形式名フレーム</summary>
		ClassID,
		/// <summary>機種名フレーム</summary>
		ModelName,
		/// <summary>セットアップ日付</summary>
		SetupDate,
		/// <summary>履歴フレーム</summary>
		History,
		/// <summary>支払情報フレーム</summary>
		PaidInfo,
		/// <summary>支払情報フレーム(ACKなし)</summary>
		PaidInfoNA,
		/// <summary>履歴なし</summary>
		NoHist,
		/// <summary>フレームエラー</summary>
		FrameError,
		/// <summary>受信タイムアウト</summary>
		TimeOut,
		/// <summary>未知のメッセージ</summary>
		UnKnownMessage,

		Null
	};

	/// <summary>
	/// 車種
	/// </summary>
	enum VehicleType
	{
		/// <summary>不明</summary>
		UnKnown = 0,
		/// <summary>普通車</summary>
		Standard = 1,
		/// <summary>大型車</summary>
		Large = 2,
		/// <summary>特殊大型車</summary>
		SuperLarge = 3,
		/// <summary>中型車</summary>
		Midium = 4,
		/// <summary>軽・二輪車</summary>
		Small = 5
	};

	// コマンドテーブル
	private byte[] bt_RES_HeartBeat = { 0x55, 0x0d };							// ETC→PC	ハートビート。 電源ON後数秒間 ETC車載器から送信される
	private byte[] bt_CMD_CommOpen	= { 0xfa, 0x0d };							// PC→ETC	接続要求
	private byte[] bt_RES_CommOK	= { 0xf0, 0x0d };							// ETC→PC	接続可能
	private byte[] bt_MES_Permit	= { 0x01, 0xc2, 0x30, 0x46, 0x32, 0x0d };	// ETC→PC	接続許可 (ACK)
	private byte[] bt_MES_CardError = { 0x01, 0xc2, 0x33, 0x46, 0x35, 0x0d };	// ETC→PC	カードエラー (ACK)
	private byte[] bt_MES_Error05 = { 0x01, 0xc2, 0x35, 0x46, 0x37, 0x0d };		// ETC→PC	エラー０５ (ACK)
	private byte[] bt_MES_CardInsert = { 0x01, 0xc2, 0x44, 0x30, 0x36, 0x0d };	// ETC→PC	カード挿入 (ACK)
	private byte[] bt_MES_CardRemove = { 0x01, 0xc2, 0x45, 0x30, 0x37, 0x0d };	// ETC→PC	カード除去 (ACK)
	private byte[] bt_MES_GateOut	= { 0x01, 0xc7, 0x41, 0x30, 0x38, 0x0d };	// ETC→PC	出口通過 (ACK)
	private byte[] bt_MES_GateIn	= { 0x01, 0xc7, 0x61, 0x32, 0x38, 0x0d };	// ETC→PC	入口通過 (ACK)
	private byte[] bt_FRMH_PaidInfo = { 0x01, 0xc5 };							// ETC→PC	支払情報(ASCII) 全11byte (ACK)
	private byte[] bt_CMD_ACK		= { 0x02, 0xc0, 0x43, 0x30, 0x0d };			// PC→ETC	ACK
	private byte[] bt_CMD_Alive		= { 0x01, 0xc6, 0x31, 0x46, 0x37, 0x0d };	// PC→ETC	通信確認
	private byte[] bt_RES_Alive		= { 0x02, 0xc2, 0x30, 0x46, 0x32, 0x0d };	// ETC→PC	通信確認OK
	private byte[] bt_CMD_CheckCard = { 0x01, 0xc6, 0x47, 0x30, 0x44, 0x0d };	// PC→ETC	カードチェック
	private byte[] bt_RES_CardNG	= { 0x02, 0xcd, 0x00, 0x43, 0x44, 0x0d };	// ETC→PC	カードなし
	private byte[] bt_RES_CardOK	= { 0x02, 0xcd, 0x01, 0x43, 0x45, 0x0d };	// ETC→PC	カードあり
	private byte[] bt_CMD_ReadHist	= { 0x01, 0xc6, 0x4c, 0x31, 0x32, 0x0d };	// PC→ETC	履歴読みだしリクエスト
	private byte[] bt_RES_HistAvail = { 0x02, 0xc1, 0x37, 0x46, 0x38, 0x0d };	// ETC→PC	履歴あり
	private byte[] bt_RES_NoHist	= { 0x02, 0xc1, 0x35, 0x46, 0x36, 0x0d };	// ETC→PC	履歴なし
	private byte[] bt_CMD_ReadNext	= { 0x01, 0xc6, 0x4d, 0x31, 0x33, 0x0d };	// PC→ETC	継続読み出しリクエスト
	private byte[] bt_RES_NoMore	= { 0x02, 0xc1, 0x38, 0x46, 0x39, 0x0d };	// ETC→PC	継続データーなし
	private byte[] bt_FRMH_History	= { 0x02, 0xe5 };							// ETC→PC	履歴データー(ASCII) 全46byte
	private byte[] bt_CMD_MachineID = { 0x01, 0xc6, 0x42, 0x30, 0x38, 0x0d };	// PC→ETC	機器番号リクエスト
	private byte[] bt_FRMH_MachineID = { 0x02, 0xcb };							// ETC→PC	機器番号(BCD) 全13byte
	private byte[] bt_CMD_ClassID	= { 0x01, 0xc6, 0x4a, 0x31, 0x30, 0x0d };	// PC→ETC	型式番号リクエスト
	private byte[] bt_FRMH_ClassID	= { 0x02, 0xe1 };							// ETC→PC	型式番号(ASCII) 全9byte
	private byte[] bt_CMD_ModelName = { 0x01, 0xc6, 0x4b, 0x31, 0x31, 0x0d };	// PC→ETC	機種名リクエスト
	private byte[] bt_FRMH_ModelName = { 0x02, 0xe2 };							// ETC→PC	機種名(ASCII) 全13byte
	private byte[] bt_CMD_SetUpDate = { 0x01, 0xc6, 0x43, 0x30, 0x39, 0x0d };	// PC→ETC	登録日リクエスト
	private byte[] bt_FRMH_SetUpDate = { 0x02, 0xcc };							// ETC→PC	登録日(BCD) 全9byte
	private byte[] bt_CMD_GetPaidInfo = { 0x01, 0xc6, 0x33, 0x46, 0x39, 0x0d };	// PC→ETC	最後の料金額リクエスト
	private byte[] bt_FRMH_PaidInfoNA = { 0x02, 0xc5 };							// ETC→PC	支払情報(ASCII) 全11byte

	/// <summary>
	/// METCクラスの初期化
	/// </summary>
	public METC()
	{
		HistoryData = new ETCHistoryData();

		// SerialPortクラスの初期化
		serial = new SerialPort();
		serial.BaudRate = 19200; serial.DataBits = 8;
		serial.StopBits = StopBits.One; serial.Handshake = Handshake.None;
		serial.Parity = Parity.Even; serial.NewLine = "\r";

		// タイムアウト用タイマーの初期化
		timer = new System.Timers.Timer(10);
		timer.Elapsed += new System.Timers.ElapsedEventHandler(timer1_Tick);
	}

	/// <summary>
	/// ETCとの通信オープン
	/// </summary>
	/// <param name="port">通信ポート名 "COM1" "COM2"など</param>
	public ETCStatus OpenPort(String port)
	{
		logfile = new StreamWriter("METCMonitor.log", true);	// 通信ログファイルの作成

		serial.PortName = port;
		try
		{
			serial.Open();
		}
		catch
		{
			return ETCStatus.TimeOut;
		}

		timer.Enabled = true;

		serial.Write( bt_CMD_CommOpen, 0, 2);               // CommOpenを送出
		timeout = 300;
		while (serial.BytesToRead < 8) {
			if (timeout == 0)                               // タイムアウト
			{
				return ETCStatus.TimeOut;
			}
			Application.DoEvents();
		}

		serial.Write(bt_CMD_ACK, 0, bt_CMD_ACK.Length);     // CMD_ACKの返信
		serial.DiscardInBuffer();

		serial.DataReceived += new SerialDataReceivedEventHandler(serial_DataReceived);

		serial.Write( bt_CMD_MachineID, 0, 6);              // 機器番号の取得
		timeout = 300;
		while (LastReceivedFrame != ETCStatus.MachineID)
		{
			if (timeout == 0)								// タイムアウト
			{
				return ETCStatus.TimeOut;
			}
			Application.DoEvents();
		}
		if (Event != null)									// 取得
		{
			Event(this, ETCStatus.MachineID, 0);
		}

		serial.Write(bt_CMD_ClassID, 0, 6);                 // 型式番号の取得
		timeout = 300;
		while (LastReceivedFrame != ETCStatus.ClassID)
		{
			if (timeout == 0)								// タイムアウト
			{
				return ETCStatus.TimeOut;
			}
			Application.DoEvents();
		}
		if (Event != null)									// 取得
		{
			Event(this, ETCStatus.ClassID, 0);
		}

		serial.Write(bt_CMD_ModelName, 0, 6);               // モデル名の取得
		timeout = 300;
		while (LastReceivedFrame != ETCStatus.ModelName)
		{
			if (timeout == 0)								// タイムアウト
			{
				return ETCStatus.TimeOut;
			}
			Application.DoEvents();
		}
		if (Event != null)									// 取得
		{
			Event(this, ETCStatus.ModelName, 0);
		}

		serial.Write(bt_CMD_SetUpDate, 0, 6);               // 登録日付の取得
		timeout = 300;
		while (LastReceivedFrame != ETCStatus.SetupDate)
		{
			if (timeout == 0)								// タイムアウト
			{
				return ETCStatus.TimeOut;
			}
			Application.DoEvents();
		}
		if (Event != null)									// 取得
		{
			Event(this, ETCStatus.SetupDate, 0);
		}

		return ETCStatus.Null;
	}

	/// <summary>
	/// 通信クローズ
	/// </summary>
	public void ClosePort()
	{
		serial.Close();
		serial.DataReceived -= new SerialDataReceivedEventHandler(serial_DataReceived);
		LinkUP = false;
		timer.Enabled = true;
		logfile.Close();
	}

	/// <summary>
	/// タイマーハンドラ
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void timer1_Tick(object sender, EventArgs e)
	{
		if (timeout > 0)
			timeout--;
	}

	/// <summary>
	/// シリアル受信ハンドラ
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void serial_DataReceived(object sender, SerialDataReceivedEventArgs e)
	{
		SerialPort sp = (SerialPort)sender;
		byte[] buf = new byte[64];
		int data, ptr;

		ptr = 0;
		try
		{
			while ((data = sp.ReadByte()) != 0x0d)			// 0x0dまで受信する
			{
				buf[ptr++] = (byte)data;
				if (ptr > 64)
				{
					ptr = 0;
					if (Event != null)						// フレームエラー
						Event(this, ETCStatus.FrameError, 0);
				}
			}
		}
		catch
		{
			return;
		}

		buf[ptr++] = 0x0d;
		string ds = "", ts = "";
		for( int i = 0; i < ptr; i++)
			ds += buf[i].ToString("x2") + " ";

		// コマンドグループ 1
		// ヘッダが 0x01のフレームには、CMD_ACKの返信が必要
		//
		if (buf[0] == 1)
		{
			// 通信許可
			//
			if (ByteComp(buf, bt_MES_Permit, 5))
			{
				LastReceivedFrame = ETCStatus.Permit;
				ts = "[PRMT]";
			}

			// カードエラー通知
			//
			else if (ByteComp(buf, bt_MES_CardError, 5))
			{
				LastReceivedFrame = ETCStatus.CardError;
				ts = "[CERR]";
				if (Event != null)
					Event( this, LastReceivedFrame, 0);

			}

			// エラー０５
			//
			else if (ByteComp(buf, bt_MES_Error05, 5))
			{
				LastReceivedFrame = ETCStatus.Error05;
				ts = "[ERR5]";
				if (Event != null)
					Event(this, LastReceivedFrame, 0);

			}

			// カード挿入通知
			//
			else if (ByteComp(buf, bt_MES_CardInsert, 5))
			{
				LastReceivedFrame = ETCStatus.CardInsert;
				ts = "[CINS]";
				if (Event != null)
					Event(this, LastReceivedFrame, 0);
			}

			// カード除去通知
			//
			else if (ByteComp(buf, bt_MES_CardRemove, 5))
			{
				LastReceivedFrame = ETCStatus.CardRemove;
				ts = "[CREM]";
				if (Event != null)
					Event(this, LastReceivedFrame, 0);
			}

			// 出口ゲート通過通知
			//
			else if (ByteComp(buf, bt_MES_GateOut, 5))
			{
				LastReceivedFrame = ETCStatus.GateOut;
				ts = "[GOUT]";
				if (Event != null)
					Event(this, LastReceivedFrame, 0);
			}

			// 入り口ゲート通過通知
			//
			else if (ByteComp(buf, bt_MES_GateIn, 5))
			{
				LastReceivedFrame = ETCStatus.GateIn;
				ts = "[GTIN]";
				if (Event != null)
					Event(this, LastReceivedFrame, 0);
			}

			// 支払通知
			//
			else if (ByteComp(buf, bt_FRMH_PaidInfo, 2))
			{
				String str = Encoding.ASCII.GetString(buf, 2, 6);	// buf[2]～[8]までをASCII変換
				LastCost = Convert.ToInt32(str);
				LastReceivedFrame = ETCStatus.PaidInfo;
				ts = "[PAID]";
				if (Event != null)
					Event(this, LastReceivedFrame, LastCost);
			}

			//
			// 未登録メッセージ
			else
			{
				LastReceivedFrame = ETCStatus.UnKnownMessage;
				ts = "[????]";
				if (Event != null)
					Event(this, LastReceivedFrame, (buf[1] << 8) | buf[2]);
			}

			sp.Write(bt_CMD_ACK, 0, bt_CMD_ACK.Length);     // CMD_ACKの返信
			LinkUP = true;
		}

		// フレームグループ 2
		// ヘッダが 0x02のフレームは受信のみでACK返信は不要
		//
		else if (buf[0] == 2)
		{
			// 機器番号フレーム
			//
			if (ByteComp(buf, bt_FRMH_MachineID, 2))
			{
				string str = "";

				for (int i = 2; i < 10; i++)
				{
					str += buf[i].ToString("X2");
				}
				machineID = str.Substring(0, 4) + "-" + str.Substring(4, 7) + "-" + str.Substring(11, 5);
				LastReceivedFrame = ETCStatus.MachineID;
				ts = "[MAID]";
			}

			// 型式番号フレーム
			//
			else if (ByteComp(buf, bt_FRMH_ClassID, 2))
			{
				classID = "";
				for (int i = 2; i < 6; i++)
				{
					classID += (char)buf[i];
				}
				LastReceivedFrame = ETCStatus.ClassID;
				ts = "[CLID]";
			}

			// 機種名フレーム
			//
			else if (ByteComp(buf, bt_FRMH_ModelName, 2))
			{
				modelName = "";
				for (int i = 2; i < 10; i++)
				{
					modelName += (char)buf[i];
				}
				LastReceivedFrame = ETCStatus.ModelName;
				ts = "[NAME]";
			}

			// 登録日付フレーム
			//
			else if (ByteComp(buf, bt_FRMH_SetUpDate, 2))
			{
				string str = "";

				for (int i = 2; i < 10; i++)
				{
					str += buf[i].ToString("X2");
				}
				setupDate = Convert.ToDateTime( str.Substring(0, 4) + "/" + str.Substring(4, 2) + "/" + str.Substring(6, 2));
				LastReceivedFrame = ETCStatus.SetupDate;
				ts = "[SDTE]";
			}

			// 履歴フレーム
			//
			else if (ByteComp(buf, bt_FRMH_History, 2))
			{
				string str = "";

				for (int i = 2; i < 48; i++)
				{
					str += (char)buf[i];
				}
				HistoryData.inGateGroup = Convert.ToInt32(str.Substring(4, 2));
				HistoryData.inGateNumber = Convert.ToInt32(str.Substring(6, 3));
				HistoryData.outGateGroup = Convert.ToInt32(str.Substring(13, 2));
				HistoryData.outGateNumber = Convert.ToInt32(str.Substring(15, 3));
				HistoryData.HistoryDate = Convert.ToDateTime(str.Substring(18, 4) + "/" + str.Substring(22, 2) + "/" + str.Substring(24, 2) + " "
					 + str.Substring(26, 2) + ":" + str.Substring(28, 2));
				HistoryData.VehicleType = (ETCHistoryData.ETCVehicleType)Convert.ToInt32(str.Substring(33, 2));
				HistoryData.cost = Convert.ToInt32(str.Substring(35, 6));

				LastReceivedFrame = ETCStatus.History;
				ts = "[HIST]";
			}

			// 履歴あり
			//
			else if (ByteComp(buf, bt_RES_HistAvail, 5))
			{
				LastReceivedFrame = ETCStatus.HistAvail;
				ts = "[HAVL]";
			}

			// 継続する履歴フレームなし
			//
			else if (ByteComp(buf, bt_RES_NoMore, 5))
			{
				LastReceivedFrame = ETCStatus.NoMore;
				ts = "[NOMO]";
			}

			// 履歴なし
			//
			else if (ByteComp(buf, bt_RES_NoHist, 5))
			{
				LastReceivedFrame = ETCStatus.NoHist;
				ts = "[NHIS]";
			}

			// カードなし
			//
			else if (ByteComp(buf, bt_RES_CardNG, 5))
			{
				LastReceivedFrame = ETCStatus.NoCard;
				ts = "[NOCD]";
				if (Event != null)
					Event(this, LastReceivedFrame, 0);
			}

			// カードあり
			//
			else if (ByteComp(buf, bt_RES_CardOK, 5))
			{
				LastReceivedFrame = ETCStatus.OnCard;
				ts = "[ONCD]";
				if (Event != null)
					Event(this, LastReceivedFrame, 0);
			}

			// 支払通知(ACK不要)
			//
			else if (ByteComp(buf, bt_FRMH_PaidInfoNA, 2))
			{
				String str = Encoding.ASCII.GetString(buf, 2, 6);
				LastCost = Convert.ToInt32(str);
				LastReceivedFrame = ETCStatus.PaidInfo;
				ts = "[PAID]";
				if (Event != null)
					Event(this, LastReceivedFrame, LastCost);
			}
		}

		else {
			// ハートビート
			// 電源ONの後数秒間送出される
			if (ByteComp(buf, bt_RES_HeartBeat, 2))
			{
				LastReceivedFrame = ETCStatus.HeartBeat;
				ts = "[HART]";
			}

			// 通信可能
			//
			else if (ByteComp(buf, bt_RES_CommOK, 2))
			{
				LastReceivedFrame = ETCStatus.CommOK;
				ts = "[COOK]";
			}
		}

		logfile.WriteLine(
			DateTime.Now.ToString("HH:mm:ss.fff") + " " + ts + " " + ds);
	}

	/// <summary>
	/// バイト配列の比較
	/// </summary>
	/// <param name="a">比較する配列 a</param>
	/// <param name="b">比較する配列 b</param>
	/// <param name="l">比較する長さ</param>
	/// <returns></returns>
	private bool ByteComp(byte[] a, byte[] b, int l)
	{
		for (int i = 0; i < l; i++)
			if (a[i] != b[i]) return false;

		return true;
	}

	/// <summary>
	/// 利用履歴の読み出し。ReadHistory() -> ReadHistory() -> [ReadNext()] * Nの順で読み出す
	/// </summary>
	/// <returns>問合わせ結果。ETCStatus.NoHistか AvailHist か History</returns>
	public ETCStatus ReadHistory()
	{
		LastReceivedFrame = ETCStatus.Null;
		serial.Write(bt_CMD_ReadHist, 0, 6);				// 履歴読みだしリクエスト
		timeout = 300;
		while (LastReceivedFrame == ETCStatus.Null)
		{
			if (timeout == 0)
			{
				return ETCStatus.TimeOut;
			}
			Application.DoEvents();							// ループ中の他コントロールに処理を渡す
		}

		return LastReceivedFrame;
	}

	/// <summary>
	/// カードチェック
	/// </summary>
	/// <returns>問合わせ結果。ETCStatus.CardNGか CardOK</returns>
	public ETCStatus CardCheck()
	{
		LastReceivedFrame = ETCStatus.Null;
		serial.Write(bt_CMD_CheckCard, 0, 6);				// カードチェック
		timeout = 300;
		while (LastReceivedFrame == ETCStatus.Null)
		{
			if (timeout == 0)
			{
				return ETCStatus.TimeOut;
			}
			Application.DoEvents();							// ループ中の他コントロールに処理を渡す
		}

		return LastReceivedFrame;
	}

	/// <summary>
	/// 継続履歴の読み出し
	/// </summary>
	/// <returns>問合わせ結果。ETCStatus.NoMoreか History</returns>
	public ETCStatus ReadNext()
	{
		LastReceivedFrame = ETCStatus.Null;
		serial.Write(bt_CMD_ReadNext, 0, 6);				// 継続読みだしリクエスト

		timeout = 300;
		while (LastReceivedFrame == ETCStatus.Null)
		{
			if (timeout == 0)
			{
				return ETCStatus.TimeOut;
			}
			Application.DoEvents();							// ループ中の他コントロールに処理を渡す
		}

		return LastReceivedFrame;
	}

	/// <summary>
	/// 料金確認
	/// </summary>
	/// <returns>問合わせ結果。ETCStatus.PaidInfo</returns>
	public ETCStatus GetPaidInfo()
	{
		LastReceivedFrame = ETCStatus.Null;
		serial.Write(bt_CMD_GetPaidInfo, 0, 6);				// 最後の利用金額リクエスト
		timeout = 300;
		while (LastReceivedFrame == ETCStatus.Null)
		{
			if (timeout == 0)
			{
				return ETCStatus.TimeOut;
			}
			Application.DoEvents();							// ループ中の他コントロールに処理を渡す
		}

		return LastReceivedFrame;
	}

}


/// <summary>
/// 履歴データークラス
/// </summary>
public class ETCHistoryData
{
	/// <summary>
	/// 車種の列挙体
	/// </summary>
	public enum ETCVehicleType : int
	{
		/// <summary>不明</summary>
		UnKnown = 0,
		/// <summary>普通車</summary>
		Standard = 1,
		/// <summary>大型車</summary>
		Large = 2,
		/// <summary>特殊大型車</summary>
		SuperLarge = 3,
		/// <summary>中型車</summary>
		Midium = 4,
		/// <summary>軽・二輪車</summary>
		Small = 5
	};

	/// <summary>履歴日付</summary>
	public DateTime HistoryDate;
	/// <summary>入口IC区分</summary>
	public int inGateGroup = 0;
	/// <summary>入口IC番号</summary>
	public int inGateNumber = 0;
	/// <summary>出口IC区分</summary>
	public int outGateGroup = 0;
	/// <summary>出口IC番号</summary>
	public int outGateNumber = 0;
	/// <summary>車種</summary>
	public ETCVehicleType VehicleType = ETCVehicleType.UnKnown;
	/// <summary>料金</summary>
	public int cost = 0;

}

